    ;;;------------------------- LICENSE NOTICE -----------------------------
    ;; This file is part of OVNI SAMURAI: an Amstrad CPC game developed by REAL BACON STUDIO
    ;;
    ;;     Copyright (C) 2020 Álvaro Sirvent, Aarón Picó, Samuel Arévalo 
    ;;
    ;;     This program is free software: you can redistribute it and/or modify
    ;;     it under the terms of the GNU General Public License as published by
    ;;     the Free Software Foundation, either version 3 of the License, or
    ;;     (at your option) any later version.
    ;;
    ;;     This program is distributed in the hope that it will be useful,
    ;;     but WITHOUT ANY WARRANTY; without even the implied warranty of
    ;;     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    ;;     GNU General Public License for more details.
    ;;
    ;;     You should have received a copy of the GNU General Public License
    ;;     along with this program.  If not, see <https://www.gnu.org/licenses/>.
    ;;
    ;; Contact REAL BACON STUDIO on:
    ;;
    ;:     Twitter:    https://twitter.com/realbaconstudio
    ;:     E-mail:
    ;:         Samuel Arévalo : arevalosamuel1608@gmail.com
    ;:         Álvaro Sirvent: alvaro231199@gmail.com
    ;;         Aarón Picó:     picoaaron@gmail.com
    ;;;----------------------------------------------------------------------
;;
;; RENDER SYSTEM
;; 

.include "cpctelera.h.s"
.area _DATA
palette: 
    .db HW_BLACK        , HW_BLUE          , HW_BRIGHT_BLUE
    .db HW_BRIGHT_GREEN   , HW_GREEN        , HW_SKY_BLUE
    .db HW_WHITE        , HW_ORANGE         , HW_PINK
    .db HW_BRIGHT_CYAN  , HW_BRIGHT_YELLOW  , HW_PASTEL_YELLOW
    .db HW_BRIGHT_CYAN  , HW_BRIGHT_YELLOW  , HW_PASTEL_YELLOW
    .db HW_PINK
;; 

.area _CODE
rendersys_init::
    ld c , #0
    call cpct_setVideoMode_asm

    cpctm_setBorder_asm HW_BLACK
    ;ld hl, #palette 
    ld hl, #_main_palette
    ld de , #16
    call cpct_setPalette_asm
    ret 

_counter: .db 0


;;
;; RENDER SPACESHIP SYSTEM
;; 
;;Sistema para pintar las entidades

;;Method to update the latest position
;;  INPUT:
;;     IX: Pointer to first entity to render 
rendersys_update_lastPosition::
    ld l , 8(ix)
    ld h , 9(ix) 
    ld a , #0x00
    ex de , hl
    ld b, 3(ix) 
    ;; Cogemos el ancho de la entidad (c)
    ld c , 2(ix) 
    call cpct_drawSolidBox_asm 
    ret
;;Method to draw all the entities in the register
;;INPUT
;;  IX: Pointer to first entity to render
;;  A: Number of entities to render
rendersys_update:: 
    _render_loop:
        ;; Guardamos a en la pila, que es lo que utilizaremos como contador
        ; push af
        ld (_counter) , a
        cp #0x00
        ret z
        ;; Setting the last position of the entity as the background color
         ;;UPDATING THE LAST POSITION OF THE ENTITY

        call check_especial_render
        
        ld a, 8(ix)
        cp #0x00
        jr nz , erase 
        ld a, 9(ix)  
        cp #0x00
        jr nz, erase
        jr not_erase
        erase: 
            ld a, e_type(ix)
            cp #5
            jp z, not_erase

            ld a, e_type(ix)
            cp #7
            jp z, not_erase

            ld a, e_type(ix)
            cp #8
            jp z, not_erase

            ld a, e_type(ix)
            cp #10
            jp z, not_erase
            
            ld a, e_type(ix)
            cp #11
            jp z, not_erase

            call rendersys_update_lastPosition
        not_erase: 
        ;; Como queremos dibujar todas las entidades del vector, haremos un bucle para recorrer cada una de ellas
        ;; Para poder dibujar una entidad, lo que necesitaremos , es su x, y, y su color 
        ld  de, #0xC000 ;;Donde meteremos el inicio de la pantalla, para calcular la posición en la que pintar con la funcion cpct_getScreenPtr_asm
        ;;Usaremos el registro ix, porque permite realizar desplazamientos, nos permite obtener a lo que apunta ix, o a +127 0 -128
        ;; Además, el registro ix, me permite cargar en cualquier registro de 8 bits
        ld c , 0(ix) ;; Aumentamos en +1 la posición a la que apunta ix, para sacar el valor de y ;; Lo que hacemos, es pasarle lo que hay en la dirección de memoria que a la que apunta  hl, al regustro b, es decir, le estamos pasando un 00
        ld b,1(ix) ;; Como el valor de Y, está en la posición a la que apunta ix, no hace falta que aumentemos
    
        call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL
        ;; Lo que usaremos para las coordenada
        ;; Dibujar un cuadrado 
        ld 8(ix), l
        ld 9(ix) , h 
        ex de , hl ;;Intercambiamos los registros En de, meteremos el puntero a memoria de vídeo, donde queremos que se pinte nuestro cuadrado. (C000 en un principio )
        ;;Cargamos el sprite de nuestra entidad 
        ld l, 6(ix) 
        ld  h, 7( ix)  ;;El patrón de colores que queremos para nuestro cuadrado
        
        ;; Cogemos el alto de la entidad (b) 
        ld b, 3(ix) 
        ;; Cogemos el ancho de la entidad (c)
        ld c , 2(ix)
        call cpct_drawSprite_asm
        
        ld a , (_counter)
        dec a
        ret z ;; BreaK  
        ld bc, #entity_size
        add ix, bc
        jr nz, _render_loop
        ret 
;;Para saber su posición, tendremos que calcularla, para eso, usaremos el getScreenPtr_asm, B = X , C = y


check_especial_render:
    ld a, e_type(ix)
    cp #0x07
    call z, eye_boss_render

    ld a, e_type(ix)
    cp #10
    call z, boss_spaceship

    ld a, e_type(ix)
    cp #11
    call z, spaceship_big_render

    ret

eye_boss_render:

    ld a, (_eye_closed)
    cp #1
    jp z, closed

    ld a, e_lives(ix)
    ld b, #40
    sub b
    jp nc, open

    ld a, e_lives(ix)
    ld b, #20
    sub b
    jp nc, damaged1

    call get_enemyEyeBoss_damaged2_sprite
    jp end_eye

    damaged1:
    call get_enemyEyeBoss_damaged1_sprite
    jp end_eye

    closed:
    call get_enemyEyeBoss_closed_sprite
    jp end_eye

    open:
    call get_enemyEyeBoss_sprite

    end_eye:
    ld 6(ix), l 
    ld  7( ix), h
    ret

boss_spaceship:
    ld a, e_lives(ix)
    ld b, #35
    sub b
    jp nc, boss_spaceship_normal

    ld a, e_lives(ix)
    ld b, #10
    sub b
    jp nc, boss_spaceship_damaged2

    call get_enemy_boss_spaceship_damaged3_sprite
    jp end_boss_spaceship


    boss_spaceship_damaged2:
    ld a, e_lives(ix)
    ld b, #25
    sub b
    jp nc, boss_spaceship_damaged1
    
    call get_enemy_boss_spaceship_damaged2_sprite
    jp end_boss_spaceship

    boss_spaceship_damaged1:
    call get_enemy_boss_spaceship_damaged1_sprite
    jp end_boss_spaceship

    boss_spaceship_normal:
    call get_enemy_boss_spaceship_sprite

    end_boss_spaceship:
    ld 6(ix), l 
    ld  7( ix), h
    ret

spaceship_big_render:
    ld a, e_lives(ix)
    ld b, #60
    sub b
    jp nc, sp_normal

    ld a, e_lives(ix)
    ld b, #20
    sub b
    jp nc, sp_damaged2

    call get_spaceship_big_damaged3_sprite
    jp end_sp


    sp_damaged2:
    ld a, e_lives(ix)
    ld b, #40
    sub b
    jp nc, sp_damaged1
    
    call get_spaceship_big_damaged2_sprite
    jp end_sp

    sp_damaged1:
    call get_spaceship_big_damaged1_sprite
    jp end_sp

    sp_normal:
    call get_spaceship_big_sprite

    end_sp:
    ld 6(ix), l 
    ld  7( ix), h
    ret
